# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from ._models_py3 import AddressValue
from ._models_py3 import AnalyzeDocumentRequest
from ._models_py3 import AnalyzeResult
from ._models_py3 import AnalyzeResultOperation
from ._models_py3 import AuthorizeCopyRequest
from ._models_py3 import AzureBlobContentSource
from ._models_py3 import AzureBlobFileListContentSource
from ._models_py3 import BoundingRegion
from ._models_py3 import BuildDocumentClassifierRequest
from ._models_py3 import BuildDocumentModelRequest
from ._models_py3 import ClassifierDocumentTypeDetails
from ._models_py3 import ClassifyDocumentRequest
from ._models_py3 import ComponentDocumentModelDetails
from ._models_py3 import ComposeDocumentModelRequest
from ._models_py3 import CopyAuthorization
from ._models_py3 import CurrencyValue
from ._models_py3 import CustomDocumentModelsDetails
from ._models_py3 import Document
from ._models_py3 import DocumentBarcode
from ._models_py3 import DocumentClassifierBuildOperationDetails
from ._models_py3 import DocumentClassifierDetails
from ._models_py3 import DocumentField
from ._models_py3 import DocumentFieldSchema
from ._models_py3 import DocumentFormula
from ._models_py3 import DocumentKeyValueElement
from ._models_py3 import DocumentKeyValuePair
from ._models_py3 import DocumentLanguage
from ._models_py3 import DocumentLine
from ._models_py3 import DocumentModelBuildOperationDetails
from ._models_py3 import DocumentModelComposeOperationDetails
from ._models_py3 import DocumentModelCopyToOperationDetails
from ._models_py3 import DocumentModelDetails
from ._models_py3 import DocumentModelSummary
from ._models_py3 import DocumentPage
from ._models_py3 import DocumentParagraph
from ._models_py3 import DocumentSelectionMark
from ._models_py3 import DocumentSpan
from ._models_py3 import DocumentStyle
from ._models_py3 import DocumentTable
from ._models_py3 import DocumentTableCell
from ._models_py3 import DocumentTypeDetails
from ._models_py3 import DocumentWord
from ._models_py3 import Error
from ._models_py3 import ErrorResponse
from ._models_py3 import GetDocumentClassifiersResponse
from ._models_py3 import GetDocumentModelsResponse
from ._models_py3 import GetOperationsResponse
from ._models_py3 import InnerError
from ._models_py3 import OperationDetails
from ._models_py3 import OperationSummary
from ._models_py3 import QuotaDetails
from ._models_py3 import ResourceDetails


from ._form_recognizer_client_enums import (
    AnalyzeResultOperationStatus,
    ContentType,
    DocumentAnalysisFeature,
    DocumentBarcodeKind,
    DocumentBuildMode,
    DocumentFieldType,
    DocumentFormulaKind,
    DocumentSignatureType,
    DocumentTableCellKind,
    FontStyle,
    FontWeight,
    LengthUnit,
    OperationKind,
    OperationStatus,
    ParagraphRole,
    SelectionMarkState,
    StringIndexType,
)
from ._patch import __all__ as _patch_all
from ._patch import *  # type: ignore # pylint: disable=unused-wildcard-import
from ._patch import patch_sdk as _patch_sdk
__all__ = [
    'AddressValue',
    'AnalyzeDocumentRequest',
    'AnalyzeResult',
    'AnalyzeResultOperation',
    'AuthorizeCopyRequest',
    'AzureBlobContentSource',
    'AzureBlobFileListContentSource',
    'BoundingRegion',
    'BuildDocumentClassifierRequest',
    'BuildDocumentModelRequest',
    'ClassifierDocumentTypeDetails',
    'ClassifyDocumentRequest',
    'ComponentDocumentModelDetails',
    'ComposeDocumentModelRequest',
    'CopyAuthorization',
    'CurrencyValue',
    'CustomDocumentModelsDetails',
    'Document',
    'DocumentBarcode',
    'DocumentClassifierBuildOperationDetails',
    'DocumentClassifierDetails',
    'DocumentField',
    'DocumentFieldSchema',
    'DocumentFormula',
    'DocumentKeyValueElement',
    'DocumentKeyValuePair',
    'DocumentLanguage',
    'DocumentLine',
    'DocumentModelBuildOperationDetails',
    'DocumentModelComposeOperationDetails',
    'DocumentModelCopyToOperationDetails',
    'DocumentModelDetails',
    'DocumentModelSummary',
    'DocumentPage',
    'DocumentParagraph',
    'DocumentSelectionMark',
    'DocumentSpan',
    'DocumentStyle',
    'DocumentTable',
    'DocumentTableCell',
    'DocumentTypeDetails',
    'DocumentWord',
    'Error',
    'ErrorResponse',
    'GetDocumentClassifiersResponse',
    'GetDocumentModelsResponse',
    'GetOperationsResponse',
    'InnerError',
    'OperationDetails',
    'OperationSummary',
    'QuotaDetails',
    'ResourceDetails',
    'AnalyzeResultOperationStatus',
    'ContentType',
    'DocumentAnalysisFeature',
    'DocumentBarcodeKind',
    'DocumentBuildMode',
    'DocumentFieldType',
    'DocumentFormulaKind',
    'DocumentSignatureType',
    'DocumentTableCellKind',
    'FontStyle',
    'FontWeight',
    'LengthUnit',
    'OperationKind',
    'OperationStatus',
    'ParagraphRole',
    'SelectionMarkState',
    'StringIndexType',
]
__all__.extend([p for p in _patch_all if p not in __all__])
_patch_sdk()