# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from ._models_py3 import AnalyzeOperationResult
from ._models_py3 import AnalyzeResult
from ._models_py3 import Appearance
from ._models_py3 import Attributes
from ._models_py3 import ComposeRequest
from ._models_py3 import CopyAuthorizationResult
from ._models_py3 import CopyOperationResult
from ._models_py3 import CopyRequest
from ._models_py3 import CopyResult
from ._models_py3 import DataTable
from ._models_py3 import DataTableCell
from ._models_py3 import DocumentResult
from ._models_py3 import ErrorInformation
from ._models_py3 import ErrorResponse
from ._models_py3 import FieldValue
from ._models_py3 import FormFieldsReport
from ._models_py3 import KeyValueElement
from ._models_py3 import KeyValuePair
from ._models_py3 import KeysResult
from ._models_py3 import Model
from ._models_py3 import ModelInfo
from ._models_py3 import Models
from ._models_py3 import ModelsSummary
from ._models_py3 import PageResult
from ._models_py3 import ReadResult
from ._models_py3 import SelectionMark
from ._models_py3 import SourcePath
from ._models_py3 import Style
from ._models_py3 import TextLine
from ._models_py3 import TextWord
from ._models_py3 import TrainRequest
from ._models_py3 import TrainResult
from ._models_py3 import TrainSourceFilter
from ._models_py3 import TrainingDocumentInfo


from ._form_recognizer_client_enums import (
    ContentType,
    FieldValueSelectionMark,
    FieldValueType,
    KeyValueType,
    Language,
    LengthUnit,
    Locale,
    ModelStatus,
    OperationStatus,
    ReadingOrder,
    SelectionMarkState,
    TextStyle,
    TrainStatus,
)
from ._patch import __all__ as _patch_all
from ._patch import *  # type: ignore # pylint: disable=unused-wildcard-import
from ._patch import patch_sdk as _patch_sdk
__all__ = [
    'AnalyzeOperationResult',
    'AnalyzeResult',
    'Appearance',
    'Attributes',
    'ComposeRequest',
    'CopyAuthorizationResult',
    'CopyOperationResult',
    'CopyRequest',
    'CopyResult',
    'DataTable',
    'DataTableCell',
    'DocumentResult',
    'ErrorInformation',
    'ErrorResponse',
    'FieldValue',
    'FormFieldsReport',
    'KeyValueElement',
    'KeyValuePair',
    'KeysResult',
    'Model',
    'ModelInfo',
    'Models',
    'ModelsSummary',
    'PageResult',
    'ReadResult',
    'SelectionMark',
    'SourcePath',
    'Style',
    'TextLine',
    'TextWord',
    'TrainRequest',
    'TrainResult',
    'TrainSourceFilter',
    'TrainingDocumentInfo',
    'ContentType',
    'FieldValueSelectionMark',
    'FieldValueType',
    'KeyValueType',
    'Language',
    'LengthUnit',
    'Locale',
    'ModelStatus',
    'OperationStatus',
    'ReadingOrder',
    'SelectionMarkState',
    'TextStyle',
    'TrainStatus',
]
__all__.extend([p for p in _patch_all if p not in __all__])
_patch_sdk()