# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from enum import Enum
from azure.core import CaseInsensitiveEnumMeta


class ContentType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Content type for upload
    """

    #: Content Type 'application/pdf'.
    APPLICATION_PDF = "application/pdf"
    #: Content Type 'image/bmp'.
    IMAGE_BMP = "image/bmp"
    #: Content Type 'image/jpeg'.
    IMAGE_JPEG = "image/jpeg"
    #: Content Type 'image/png'.
    IMAGE_PNG = "image/png"
    #: Content Type 'image/tiff'.
    IMAGE_TIFF = "image/tiff"
    #: Content Type 'application/json'.
    APPLICATION_JSON = "application/json"

class FieldValueSelectionMark(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Selection mark value.
    """

    SELECTED = "selected"
    UNSELECTED = "unselected"

class FieldValueType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Semantic data type of the field value.
    """

    STRING = "string"
    DATE = "date"
    TIME = "time"
    PHONE_NUMBER = "phoneNumber"
    NUMBER = "number"
    INTEGER = "integer"
    ARRAY = "array"
    OBJECT = "object"
    SELECTION_MARK = "selectionMark"
    COUNTRY_REGION = "countryRegion"

class KeyValueType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Semantic data type of the key value element.
    """

    STRING = "string"
    SELECTION_MARK = "selectionMark"

class Language(str, Enum, metaclass=CaseInsensitiveEnumMeta):

    AF = "af"
    AST = "ast"
    BI = "bi"
    BR = "br"
    CA = "ca"
    CEB = "ceb"
    CH = "ch"
    CO = "co"
    CRH = "crh"
    CS = "cs"
    CSB = "csb"
    DA = "da"
    DE = "de"
    EN = "en"
    ES = "es"
    ET = "et"
    EU = "eu"
    FI = "fi"
    FIL = "fil"
    FJ = "fj"
    FR = "fr"
    FUR = "fur"
    FY = "fy"
    GA = "ga"
    GD = "gd"
    GIL = "gil"
    GL = "gl"
    GV = "gv"
    HNI = "hni"
    HSB = "hsb"
    HT = "ht"
    HU = "hu"
    IA = "ia"
    ID = "id"
    IT = "it"
    IU = "iu"
    JA = "ja"
    JV = "jv"
    KAA = "kaa"
    KAC = "kac"
    KEA = "kea"
    KHA = "kha"
    KL = "kl"
    KO = "ko"
    KU = "ku"
    KW = "kw"
    LB = "lb"
    MS = "ms"
    MWW = "mww"
    NAP = "nap"
    NL = "nl"
    NO = "no"
    OC = "oc"
    PL = "pl"
    PT = "pt"
    QUC = "quc"
    RM = "rm"
    SCO = "sco"
    SL = "sl"
    SQ = "sq"
    SV = "sv"
    SW = "sw"
    TET = "tet"
    TR = "tr"
    TT = "tt"
    UZ = "uz"
    VO = "vo"
    WAE = "wae"
    YUA = "yua"
    ZA = "za"
    ZH_HANS = "zh-Hans"
    ZH_HANT = "zh-Hant"
    ZU = "zu"

class LengthUnit(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The unit used by the width, height and boundingBox properties. For images, the unit is "pixel".
    For PDF, the unit is "inch".
    """

    PIXEL = "pixel"
    INCH = "inch"

class Locale(str, Enum, metaclass=CaseInsensitiveEnumMeta):

    EN_AU = "en-AU"
    EN_CA = "en-CA"
    EN_GB = "en-GB"
    EN_IN = "en-IN"
    EN_US = "en-US"

class ModelStatus(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Status of the model.
    """

    CREATING = "creating"
    READY = "ready"
    INVALID = "invalid"

class OperationStatus(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Status of the queued operation.
    """

    NOT_STARTED = "notStarted"
    RUNNING = "running"
    SUCCEEDED = "succeeded"
    FAILED = "failed"

class ReadingOrder(str, Enum, metaclass=CaseInsensitiveEnumMeta):

    BASIC = "basic"
    NATURAL = "natural"

class SelectionMarkState(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """State of the selection mark.
    """

    SELECTED = "selected"
    UNSELECTED = "unselected"

class TextStyle(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The text line style name, including handwriting and other.
    """

    OTHER = "other"
    HANDWRITING = "handwriting"

class TrainStatus(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Status of the training operation.
    """

    SUCCEEDED = "succeeded"
    PARTIALLY_SUCCEEDED = "partiallySucceeded"
    FAILED = "failed"
